"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameRecordCard = void 0;
const tslib_1 = require("tslib");
const request_1 = require("../utils/request");
const base_1 = require("./base");
const constants_1 = require("../constants/constants");
const alias_1 = require("../utils/alias");
const validator_1 = require("../utils/validator");
const mergeOptions_1 = tslib_1.__importDefault(require("../utils/mergeOptions"));
class GameRecordCard extends base_1.BaseRoute {
    constructor(options) {
        super(options);
        Object.defineProperty(this, "defaultOptions", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.defaultOptions = options?.defaultOptions;
    }
    /**
     * @param {string} uid HoYoLab Account ID
     */
    async fetch(uid, options) {
        if (this.cache?.has(uid))
            return this.cache.get(uid);
        const optionsToUse = (0, mergeOptions_1.default)(options, this.cookieManager, this.defaultOptions);
        if (!(0, validator_1.basicValidator)(uid, optionsToUse)) {
            throw new Error("No UID or Cookie provided");
        }
        const { language, cookie } = optionsToUse;
        const instance = new request_1.request({
            route: constants_1.Genshin_Hoyolab_API_URL,
        });
        instance.setLanguage(language);
        const res = await instance.get("getGameRecordCard", {
            Cookie: cookie,
        }, {
            uid: uid,
        });
        const { data } = res;
        (0, alias_1.removeFromArrayObject)(data.list, ["h5_data_switches", "data_switches"]);
        const returnData = {
            list: data.list,
            currecnt: data.list[0],
        };
        if (this.cache) {
            this.cache.set(uid, returnData);
        }
        return returnData;
    }
}
exports.GameRecordCard = GameRecordCard;
