"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Charcters = void 0;
const tslib_1 = require("tslib");
const request_1 = require("../utils/request");
const getServer_1 = require("../utils/getServer");
const base_1 = require("./base");
const validator_1 = require("../utils/validator");
const mergeOptions_1 = tslib_1.__importDefault(require("../utils/mergeOptions"));
class Charcters extends base_1.BaseRoute {
    constructor(options) {
        super(options);
        Object.defineProperty(this, "defaultOptions", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.defaultOptions = options?.defaultOptions;
    }
    /**
     * @param {string} uid Genshin Impact UID
     */
    async fetch(uid, options) {
        if (this.cache?.has(uid))
            return this.cache.get(uid);
        const optionsToUse = (0, mergeOptions_1.default)(options, this.cookieManager, this.defaultOptions);
        if (!(0, validator_1.basicValidator)(uid, optionsToUse)) {
            throw new Error("No UID or Cookie provided");
        }
        const { language, cookie } = optionsToUse;
        const instance = new request_1.request({
            withDS: true,
            withExtraHeaders: true,
        });
        instance.setLanguage(language);
        const res = await instance.post("character", {
            Cookie: cookie,
        }, {
            role_id: uid,
            server: (0, getServer_1.checkServerRegion)(uid),
        });
        const { data } = res;
        if (this.cache) {
            this.cache.set(uid, data.avatars);
        }
        return data.avatars;
    }
    /**
     * @param {number} characterId The avatar's id
     */
    async fetchAvatarInfo(characterId, options) {
        const optionsToUse = (0, mergeOptions_1.default)(options, this.cookieManager, this.defaultOptions);
        if (!(0, validator_1.getAvatarValidator)(characterId, optionsToUse)) {
            throw new Error("No UID or Cookie provided");
        }
        const { language, cookie } = optionsToUse;
        const instance = new request_1.request({
            withDS: true,
            withExtraHeaders: true,
        });
        instance.setLanguage(language);
        const res = await instance.post("avatarBasicInfo", {
            Cookie: cookie,
        }, {
            character_ids: [characterId],
        });
        const { data } = res;
        return data.avatars[0];
    }
}
exports.Charcters = Charcters;
